      subroutine zbjy(x,m,nstop,fnu,
     & 							bjr,byr,cjr,cji)
c
c *********************************************************************
c
c  Subroutine zbjy gets J & Y Bessel functions for use in
c  sphere (fnu=0.50) or cylinder (fnu=0.0d0) scattering calculations.
c  Scaled or nonscaled functions for argument z = m*x are returned
c  depending
c  on the magnitude of the product of the size parameter and the complex
c  refractive index (zabs(z)). Nonscaled functions are returned if the
c  imaginary part of the refractive index is zero. Nonscaled functions are
c  returned for argument x.
c
c       Merrill Milham      >>> version: 2.0 <<<        JANUARY 1994
C
c  inputs:
c       x = the size parameter of the cylinder.(real*8)
c       m = the complex refractive index, n - ik.(complex*16)
c   nstop = the highest order of the Bessel functions.(integer)
c     fnu = 0.5d0 for sphere calculations or
c           0.0d0 for cylinder calculations. (real*8)
c
c  outputs:
c
c   bjr = real part of J(x) Bessel functions.(array: real*8)
c   byr = real part of Y(x) Bessel functions.(array: real*8)
c   cjr = real part of J(m*x) Bessel functions.(array: real*8)
c   cji = imag. part of J(m*x) Bessel functions.(array: real*8)
c
c subroutines used:
c
c           zbesj - returns J Bessel functions
c           zbesy - returns Y Bessel functions
c
c Reference: D. E. Amos, "Algorthim 644: A Portable Package for Bessel
c            Functions of a Complex Argument and Nonnegative Order,"
c            ACM Transcations on Mathematical Software, 12,265-273(1986).
c
c *************************************************************************
c
      implicit none
cthw      automatic cwrkr,cwrki,bji,byi
*
      integer al
      real*8 zero,xll,two
      parameter (al=5100,zero=0.0d0,xll=1.d-1,two=2.d0) 
*
      real*8 bjr(1) ,byr(1)
c
c page 31 
c
      real*8 x,fnu,cjr(1),cji(1)
      real*8 zr,zi,d1mach
      real*8 cwrkr(al),cwrki(al)
      real*8 r1m5,elim,aa,alim
      complex*16 m,z
      integer kode,ierr,nz,nstop,n
      integer k1,i1mach,k2,k
      logical zflag,eflg1,eflg2
c
      kode=1
c
      call zbesj(x,zero,fnu,kode,nstop,bjr,cji,nz,ierr) 
*
      eflg1=ierr.eq.0
      eflg2=nz.eq.0
      if (eflg1.and.eflg2) then
            continue
        else if (.not.eflg2.and.eflg1) then
                  nstop=nstop-nz
                  print*,'zbesj error: ierr =',ierr,'nz =',nz
                    else
              print*,'inputs =',x,zero,fnu,kode,nstop
              print*,'zbesj called from subroutine zbjy'
      end if
c
      call zbesy(x,zero,fnu,kode,nstop,byr,cji,nz,cwrkr,cwrki,ierr) 
*
      eflg1=ierr.eq.0
      eflg2=nz.eq.0
      if (eflg1.and.eflg2) then
            continue
            else if (.not.eflg2.and.eflg1) then
                        nstop=nstop-nz
                        print*,'zbesy error: ierr =',ierr,'nz =',nz
            continue
                    else
                print*,'zbesy error: ierr =',ierr,'nz=',nz
                print*,'inputs =',x,zero,fnu,kode,nstop
                print*,'zbesy called from subroutine zbjy'
      end if
C
      z=m*x
      zr=dble(z)
      zi=dimag(z)
      if(zi.ne.zero) then 
*
      k1 = i1mach(15)
      k2 = i1mach(16)
      r1m5 = d1mach(5)
      k = min0(iabs(k1),iabs(k2))
      elim = 2.303d0*(dble(float(k))*r1m5-3.0d0)
      k1 = i1mach(14) - 1
c
c page 32 
c
      aa = r1m5*dble(float(k1))
      aa = aa*2.303d0
      alim = elim + dmax1(-aa,-41.45d0) 
*
            if(dabs(zi).gt.alim) kode=2
                                    else
           continue
      end if
      call zbesj(zr,zi,fnu,kode,nstop,cjr,cji,nz,ierr)
*
      eflg1=ierr.eq.0
      eflg2=nz.eq.0
      if (eflg1.and.eflg2) then
            continue
            else if (.not.eflg2.and.eflg1) then
                        nstop=nstop-nz
                        print*,'zbesj error: ierr =',ierr,'nz =',nz
            continue
                        else
                print*,'zbesj error: ierr =',ierr,'nz =',nz
                print*,'inputs =',zr,zi,fnu,kode,nstop
                print*,'zbesj called from subroutine zbjy'
      end if
c
      zflag=dabs(zi).lt.two*d1mach(1).and.x.lt.xll
      if (.not.zflag) then
            continue
                    else
            do 100 n=1,nstop
                cji (n)=zero
100         continue
      end if 
*
      zflag=dabs(zr).lt.two*d1mach(1).and.x.lt.xll
      if (.not.zflag) then
            continue
                      else
            cji(1)=zero
            do 200 n=2,nstop
                    if(mod(n,2).eq.0) then
                        cjr(n)=zero
                                       else
                        cji(n)=zero
                    end if
200     continue
      end if
c
      return
c
      end
c